/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnIterationBean.java,v 1.1 2004/07/30 14:57:56 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnIterationPK;
import hero.interfaces.BnIterationValue;
import hero.interfaces.BnIterationLightValue;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;


 /**
 * The Entity bean represents a BnIteration
 *
 * @ejb:bean name="BnIteration"
 *           display-name="Iteration"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnIteration"
 *           local-jndi-name="ejb/hero/BnIteration_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"  
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 * 
 *
 * @ejb.pk generate="true" method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnIteration"
 *     jndi-name="ejb/hero/BnIteration"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnIteration"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(p) FROM BnIteration p"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByProject(int projectId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnIterationLocal findByFromTo(java.lang.String from, java.lang.String to, int projectId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?3 AND p.fromNode=?1 AND p.toNode=?2"
 *     transaction-type="Supports" 
 *
 * @ejb:finder signature="hero.interfaces.BnIterationLocal findByFrom(int projectId, java.lang.String from)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1 AND p.fromNode=?2"
 *     transaction-type="Supports"
 * 
 * @ejb:finder signature="hero.interfaces.BnIterationLocal findByTo(int projectId, java.lang.String to)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1 AND p.toNode=?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnIteration"
 * 
 * @ejb.value-object
 *    match="light"
 *    name="BnIterationLight"
 *
 *
 * @jboss:table-name table-name="BnIteration"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/
public abstract class BnIterationBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnIteration  id.
   *
   * @return Returns an int representing the id of this BnIteration.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"   
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the Iteration id.
   *
   * @param pId The id of this NodeIteration. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   

    /** Retrieve the BnProject.
     *
     * @return Returns a BnProjectLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="projectIte" role-name="iteration-by" cascade-delete="yes"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnProjectLocal getBnProject();
    
    /**
     * Set the BnProject.
     *
     * @ejb:interface-method view-type="local"
     * @param pProject The BnProject of this Property.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject( BnProjectLocal pProject );


   /**
   * Retrieve the from node.
   *
   * @return Returns an String representing the from node value of this Iteration.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="from"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getFromNode();
   
   /**
   * Set the from node value.
   * @ejb:interface-method view-type="both"
   * @param from The from node of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setFromNode( String from );

   /**
   * Retrieve the to node.
   *
   * @return Returns an String representing the to node value of this Iteration.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="to"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getToNode();
   
   /**
   * Set the to node value.
   * @ejb:interface-method view-type="both"
   * @param to The to node value of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setToNode( String to );
   
   /**
   * Retrieve the condition value.
   *
   * @return Returns an String representing the condition value of this Iteration.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="condition"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getCondition();
   
   /**
   * Set the condition value.
   * @ejb:interface-method view-type="both"
   * @param condition The condition value of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setCondition( String condition );

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnIterationValue getBnIterationValue();
    
	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Supports"
	 */
	public abstract BnIterationLightValue getBnIterationLightValue();
    
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnIterationValue(BnIterationValue v);
    
	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnIterationLightValue(BnIterationLightValue v);




   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a NodeIteration based on the supplied Property Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnIterationPK ejbCreate(BnIterationLightValue nd )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnIterationLightValue(nd);

	   try {
			 BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			 this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.ITERATION));
		
	    } catch (BonitaServiceException e) {
			 throw new CreateException(e.getMessage());
		}
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a NodeIteration based on the supplied  key  and value.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnIterationPK ejbCreate(String from, String to, String condition)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setFromNode(from);
       this.setToNode(to);
	   this.setCondition(condition);

	   try {
			 BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			 this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.ITERATION));
		
	   } catch (BonitaServiceException e) {
			 throw new CreateException(e.getMessage());
	   }

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }
   
}
