/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnRoleMapperBean.java,v 1.1 2004/07/30 14:57:56 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnRoleMapperPK;
import hero.interfaces.BnRoleMapperValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnRoleMapper
 *
 * @author Marc Blachon
 * @version $Revision: 1.1 $
 *
 * @ejb:bean name="BnRoleMapper"
 *           display-name="BnRole Mapper Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnRoleMapper"
 *           local-jndi-name="ejb/hero/BnRoleMapper_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"    
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 * 
 *
 * @ejb.pk generate="true" method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnRoleMapper"
 *     jndi-name="ejb/hero/BnRoleMapper"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnRoleMapper"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(n) FROM BnRoleMapper n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByRole(int RoleId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnRoleMapper p Where p.bnRole.id = ?1"
 *     transaction-type="Supports"
 * 
 * @ejb:finder signature="hero.interfaces.RoleMapperLocal findByName(java.lang.String name)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnRoleMapper p Where p.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnRoleMapper"
 *
 * @jonas.ejb-relation
 *        ejb-relation-name="mapper-role"
 *        ejb-relationship-role-name1="mapper2role"
 *        foreign-key-jdbc-name1="role_id"
 * *      key-jdbc-name1="id"
 *
 * @jboss:table-name table-name="BnRoleMapper"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/
public abstract class BnRoleMapperBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnRoleMapper's id.
   *
   * @return Returns an int representing the id of this BnRoleMapper.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the Mapper's id.
   *
   * @param pId The id of this Mapper. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   
   
   /** Retrieve the role associated with the mapper.
	*
	* @return Returns a String representing the Name of the role.
	* @ejb:interface-method view-type="both"
	* @ejb:relation name="mapper-role" role-name="mapper2role" cascade-delete="yes"
	* @jboss:auto-key-fields
	* @jonas:automatic-pk="true"
	* @ejb:transaction type="Supports"
	**/
    public abstract BnRoleLocal getBnRole();
    
   
    /**
     * Set the BnRole.
     *
     * @ejb:interface-method view-type="local"
     * @param pRole The BnRole of this Mapper.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnRole( BnRoleLocal pRole );

   /**
   * Retrieve the Name value.
   *
   * @return Returns an String representing the Mapper Name value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the Name value.
   * @ejb:interface-method view-type="both"
   * @param key The Name of this Mapper.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String mapperName );


   /**
   * Retrieve the Type value.
   *
   * @return Returns an int representing the Type value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();
   
   /**
   * Set the Type value.
   * @ejb:interface-method view-type="both"
   * @param value The Type value of this Mapper.
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int mapperType );

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnRoleMapperValue getBnRoleMapperValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnRoleMapperValue(BnRoleMapperValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a Mapper based on the supplied Mapper Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnRoleMapperPK ejbCreate(BnRoleMapperValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnRoleMapperValue(prp);


       try {
		  BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
		  this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.ROLE_MAPPER));
		
	   } catch (BonitaServiceException e) {
		  throw new CreateException(e.getMessage());
 	   }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a Mapper based on the supplied  parameters.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnRoleMapperPK ejbCreate(String mapperName, int mapperType)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(mapperName);
       this.setType(mapperType);

	   try {
			 BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			 this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.ROLE_MAPPER));
		
	   } catch (BonitaServiceException e) {
			 throw new CreateException(e.getMessage());
	   }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }
}
